@@ -1,7 +1,7 @@
 /* Strip Club - Online/Offline Comic Reader/Archiver
  *
  * Copyright notice for this file:
- *  Copyright (C) 2004 Benjamin Cutler
+ *  Copyright (C) 2004,2005 Benjamin Cutler
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,20 +22,25 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#ifdef _WIN32
+#ifdef WIN32
 #include <sys/stat.h>
+#define main _realMain		// Because without this, it enters at main(), and doesn't initialize properly
+#elif !defined(__APPLE__)
+#include <X11/xpm.h>
+#include "stripclub.xpm"
 #endif
 #include <pcre.h>
-#include <SDL/SDL_net.h>
 #include <FL/Fl_Box.H>
 #include <FL/Fl_Preferences.H>
 #include <FL/Fl_Text_Buffer.H>
 #include <FL/fl_ask.H>
+#include <FL/x.H>
 #include "load.h"
 #include "url.h"
 #include "http.h"
 #include "html.h"
 
+
 char *CurComic;
 Fl_Preferences *Prefs = NULL, *CurComicG = NULL;
 cInterface Interface;
@@ -49,36 +54,16 @@ void EditComic(Fl_Widget *w, void *Comic);
 void OpenStatsWindow(Fl_Widget *w, void *Comic);
 void DonateToComic(Fl_Widget *w, void *Comic);
 void OpenStore(Fl_Widget *w, void *Comic);
+void SetIcon(Fl_Window *w);
 
 // import.cpp
 void ExportComic(Fl_Widget *w, void *Comic);
 void UpdateComics();
 
 int main(int argc, char **argv) {
-	Fl_Text_Buffer TextOutputBuffer, UpstreamOutputBuffer, DownstreamOutputBuffer, DebugOutputBuffer;
+	Fl_Text_Buffer TextOutputBuffer, ErrorOutputBuffer, UpstreamOutputBuffer, DownstreamOutputBuffer, DebugOutputBuffer;
 	int oldw = 0, oldh = 0;
 
-#ifdef _WIN32
-	{
-		struct stat filestat;
-		if (stat("stripclub.exe", &filestat)) {	// Can't find the executable, so the working dir must be wrong (sanity check)
-			MOut("Strip Club should not be run from any directory but the location of the executable. Working directory is pointing to the wrong place, perhaps?\n");
-			return 1;
-		}
-	}
-#endif
-
-	if(SDL_Init(0)==-1) {
-		Out("SDL_Init: %s\n", SDL_GetError());
-		exit(1);
-	}
-	if(SDLNet_Init()==-1) {
-		Out("SDLNet_Init: %s\n", SDLNet_GetError());
-		exit(2);
-	}
-
-	debug = fopen(GetCacheFileName("debug.log", true, "debug"), "w");
-
 	Quit = false;
 
 	CurComic = CurLinks.FirstLink = CurLinks.PrevLink = CurLinks.CurLink = CurLinks.NextLink = CurLinks.LatestLink = NULL;
@@ -88,6 +73,7 @@ int main(int argc, char **argv) {
 	Prefs = new Fl_Preferences(Fl_Preferences::USER, "racercksproductions", "stripclub");
 
 	TextOutput->buffer(&TextOutputBuffer);
+	ErrorOutput->buffer(&ErrorOutputBuffer);
 	UpstreamOutput->buffer(&UpstreamOutputBuffer);
 	DownstreamOutput->buffer(&DownstreamOutputBuffer);
 	DebugOutput->buffer(&DebugOutputBuffer);
@@ -129,15 +115,34 @@ int main(int argc, char **argv) {
 
 	Interface.Make();
 
-	Interface.MainWindow->show();
+	SetIcon(Interface.MainWindow);
+	SetIcon(OutputWindow);
+
+	Interface.MainWindow->show(argc, argv);		// Have to use the (argc, argv) format in order to get the icon to show
 
 	{
 		int remx, remy, remw, remh;
-		Prefs->get("remx", remx, 4);
-		Prefs->get("remy", remy, 4);
+		Prefs->get("remx", remx, 40);
+		Prefs->get("remy", remy, 40);
 		Prefs->get("remw", remw, 640);
 		Prefs->get("remh", remh, 400);
 		Interface.MainWindow->resize(remx, remy, remw, remh);
+		Interface.MainMenu->resize(0, 0, 144, 20);
+		Interface.OutputURL->resize(144, 0, remw - 208, 20);
+		Interface.OutputOffline->resize(remw - 64, 0, 64, 20);
+		Interface.ScrollBox->resize(2, 22, remw - 4, remh - 68);
+		Interface.DownloadProgress->resize(2, remh - 44, remw - 286, 20);
+		Interface.StatusBar->resize(2, remh - 22, remw - 286, 20);
+		Interface.PrevComicButton->resize(remw - 282, remh - 44, 112, 20);
+		Interface.GalleryButton->resize(remw - 170, remh - 44, 56, 20);
+		Interface.CancelButton->resize(remw - 170, remh - 44, 56, 20);
+		Interface.NextComicButton->resize(remw - 114, remh - 44, 112, 20);
+		Interface.FirstButton->resize(remw - 282, remh - 22, 56, 20);
+		Interface.PrevButton->resize(remw - 226, remh - 22, 56, 20);
+		Interface.RefreshButton->resize(remw - 170, remh - 22, 56, 20);
+		Interface.NextButton->resize(remw - 114, remh - 22, 56, 20);
+		Interface.LatestButton->resize(remw - 58, remh - 22, 56, 20);
+		Interface.MainWindow->redraw();
 	}
 
 	{
@@ -152,6 +157,48 @@ int main(int argc, char **argv) {
 		Prefs->set("warning", 1);
 	}
 
+	if (Prefs->groupExists("hosts")) {	// Migrate old "hosts" group to a seperate prefs file
+		Fl_Preferences *HostCacheOld, *HostCacheNew;
+		HostCacheOld = new Fl_Preferences(Prefs, "hosts");
+		HostCacheNew = new Fl_Preferences(Fl_Preferences::USER, "racercksproductions", "hosts");
+		for(int i = 0; i < HostCacheOld->entries(); i++) {
+			char *Buf;
+			HostCacheOld->get(HostCacheOld->entry(i), Buf, "");
+			DbgOut("Moving: %s\n", Buf);
+			HostCacheNew->set(HostCacheOld->entry(i), Buf);
+			free(Buf);
+		}
+		delete HostCacheOld;
+		delete HostCacheNew;
+		Prefs->deleteGroup("hosts");
+	}
+
+	{
+		int DelHosts;
+		if (!Prefs->get("deletehosts", DelHosts, 0)) {
+			if (fl_ask("As of version 0.9 Strip Club now supports\nthe option of deleting the Host cache on startup\nto avoid stale entries. This option is\noff by default.\n\nWould you like to turn it on?")) {
+				Prefs->set("deletehosts", 1);
+				DelHosts = 1;
+			} else {
+				Prefs->set("deletehosts", 0);
+			}
+		}
+		if(DelHosts) {
+			Fl_Preferences *HostCache = new Fl_Preferences(Fl_Preferences::USER, "racercksproductions", "hosts");
+			while (HostCache->entries()) {
+				DbgOut("Deleting Host: %s\n", HostCache->entry(0));
+				HostCache->deleteEntry(HostCache->entry(0));
+			}
+			delete HostCache;
+		}
+	}
+
+	{
+		int Timeout;
+		Prefs->get("timeout", Timeout, 15);
+		ValueTimeout->value(Timeout);
+	}
+
 	InitMenu();
 
 	while (Interface.MainWindow->shown() && !Quit) {
@@ -161,25 +208,23 @@ int main(int argc, char **argv) {
 			Interface.MainMenu->resize(0, 0, 144, 20);
 			Interface.OutputURL->resize(144, 0, oldw - 208, 20);
 			Interface.OutputOffline->resize(oldw - 64, 0, 64, 20);
-			Interface.ScrollBox->resize(2, 22, oldw - 4, oldh - 46);
-			Interface.Progress->resize(2, oldh - 22, oldw - 288, 20);
-			Interface.FirstButton->resize(oldw - 284, oldh - 22, 56, 20);
-			Interface.PrevButton->resize(oldw - 228, oldh - 22, 56, 20);
-			Interface.RefreshButton->resize(oldw - 172, oldh - 22, 56, 20);
-			Interface.NextButton->resize(oldw - 116, oldh - 22, 56, 20);
-			Interface.LatestButton->resize(oldw - 60, oldh - 22, 56, 20);
+			Interface.ScrollBox->resize(2, 22, oldw - 4, oldh - 68);
+			Interface.DownloadProgress->resize(2, oldh - 44, oldw - 286, 20);
+			Interface.StatusBar->resize(2, oldh - 22, oldw - 286, 20);
+			Interface.PrevComicButton->resize(oldw - 282, oldh - 44, 112, 20);
+			Interface.GalleryButton->resize(oldw - 170, oldh - 44, 56, 20);
+			Interface.CancelButton->resize(oldw - 170, oldh - 44, 56, 20);
+			Interface.NextComicButton->resize(oldw - 114, oldh - 44, 112, 20);
+			Interface.FirstButton->resize(oldw - 282, oldh - 22, 56, 20);
+			Interface.PrevButton->resize(oldw - 226, oldh - 22, 56, 20);
+			Interface.RefreshButton->resize(oldw - 170, oldh - 22, 56, 20);
+			Interface.NextButton->resize(oldw - 114, oldh - 22, 56, 20);
+			Interface.LatestButton->resize(oldw - 58, oldh - 22, 56, 20);
 			Interface.MainWindow->redraw();
 		}
-		if (!OutputWindow->shown()) {
-			OutputWindow->show();
-		}
 		Fl::wait();
 	}
 
-	SDLNet_Quit();
-
-	SDL_Quit();
-
 	if(CurComicG) {
 		delete CurComicG;
 	}
@@ -194,8 +239,6 @@ int main(int argc, char **argv) {
 	delete OutputWindow;
 	delete Prefs;
 
-	fclose(debug);
-
 	remove(GetCacheFileName(TEMPHTML, true, "temp"));
 	remove(GetCacheFileName(TEMPTXT, true, "temp"));
 #ifndef NOGZIP
@@ -212,6 +255,7 @@ void InitMenu() {
 	char buf[400], *name, *category;
 	Interface.MainMenu->menu(cInterface::menu_MainMenu); 		// Reset the menu in case we're reloading
 	if(!comics) {
+		Interface.MainWindow->show();
 		MOut("No comics found. Make sure you import/create some.\n");
 		if (fl_ask("Run updater to download .comic files off the Strip Club site?")) {
 			UpdateComics();
@@ -220,16 +264,16 @@ void InitMenu() {
 	}
 	if (groupnames) {
 		for (i = 0; groupnames[i]; i++) {
-			delete groupnames[i];
+			free(groupnames[i]);
 		}
-		delete groupnames;
+		free(groupnames);
 	}
-	groupnames = new char*[comics + 1];
+	groupnames = (char**)malloc(sizeof(char *) * (comics + 1));
 	groupnames[comics] = 0;			// To mark the end of the array
 	for (i = 0; i < comics; i++) {
-		clonestr(Prefs->group(i), (groupnames + i));
+		groupnames[i] = strdup(Prefs->group(i));
 		if(strlen(groupnames[i])) {
-			Out("Loading Comic: %s\n", groupnames[i]);
+			DbgOut("Loading Comic: %s\n", groupnames[i]);
 			delete TempGroup;
 			TempGroup = new Fl_Preferences(Prefs, groupnames[i]);
 			if (TempGroup->entryExists("name")) {
@@ -257,6 +301,7 @@ void InitMenu() {
 		}
 	}
 	delete TempGroup;
+	Out("Loaded %d comic(s)\n", comics);
 }
 
 void AddComic() {
@@ -266,11 +311,11 @@ void AddComic() {
 	if (!Buffer || !strlen(Buffer)) {
 		return;
 	}
-	if (!strcmp(Buffer, "hosts")) {
-		MOut("Error: \"hosts\" is a reserved value, please select another.\n");
+	if (!strcmp(Buffer, "hosts") || !strcmp(Buffer, "stripclub")) {
+		MOut("Error: \"%s\" is a reserved value, please select another.\n", Buffer);
 		return;
 	}
-	clonestr(Buffer, &GroupName);
+	GroupName = strdup(Buffer);
 	if (Prefs->groupExists(GroupName)) {
 		if (!fl_ask("Comic already exists! Edit?")) {
 			free(GroupName);
@@ -284,6 +329,11 @@ void AddComic() {
 void EditComic(Fl_Widget *w, void *Comic) {
 	Fl_Preferences *EditComic = new Fl_Preferences(Prefs, (char *)Comic);
 	char Buffer[400];
+	if (EditComic->entryExists("skip")) {
+		CheckSkip->value(1);
+	} else {
+		CheckSkip->value(0);
+	}
 	EditComic->get("name", Buffer, "New Comic", 399);
 	InputName->value(Buffer);
 	EditComic->get("baseurl", Buffer, "http://", 399);
@@ -318,6 +368,8 @@ void EditComic(Fl_Widget *w, void *Comic) {
 		InputUserAgent->deactivate();
 		ChoiceUserAgent->deactivate();
 	}
+	EditComic->get("imgfilepattern", Buffer, "", 399);
+	InputImgFilePattern->value(Buffer);
 	EditComic->get("category", Buffer, "Unsorted", 399);
 	InputCategory->value(Buffer);
 	EditComic->get("donate", Buffer, "", 399);
@@ -331,6 +383,11 @@ void EditComic(Fl_Widget *w, void *Comic) {
 		EditComic = NULL;
 		Prefs->deleteGroup((char *)Comic);
 	} else if (strcmp("cancel", InputName->value())) {		// We did NOT click "Cancel"
+		if (CheckSkip->value()) {
+			EditComic->set("skip", 1);
+		} else {
+			EditComic->deleteEntry("skip");
+		}
 		EditComic->set("name", InputName->value());
 		EditComic->set("baseurl", InputBaseURL->value());
 		EditComic->set("firsturl", InputFirstURL->value());
@@ -357,6 +414,11 @@ void EditComic(Fl_Widget *w, void *Comic) {
 		} else {									// Not checked
 			EditComic->set("useragent", InputUserAgent->value());
 		}
+		if (strlen(InputImgFilePattern->value())) {
+			EditComic->set("imgfilepattern", InputImgFilePattern->value());
+		} else {
+			EditComic->deleteEntry("imgfilepattern");
+		}
 		EditComic->set("category", InputCategory->value());
 		if (strlen(InputDonate->value())) {
 			EditComic->set("donate", InputDonate->value());
@@ -421,6 +483,7 @@ void OpenStatsWindow(Fl_Widget *w, void *Comic) {
 void ChangeSettings() 
 {
 	char *buf;
+	int DelHosts, Timeout;
 	Prefs->get("browser", buf, DEFAULT_BROWSER);
 	InputBrowser->value(buf);
 	free(buf);
@@ -429,6 +492,10 @@ void ChangeSettings()
 	free(buf);
 	ChoiceDownload->value((int)Offline);
 	CheckDebug->value(Debug);
+	Prefs->get("deletehosts", DelHosts, 0);
+	CheckEraseHosts->value(DelHosts);
+	Prefs->get("timeout", Timeout, 15);
+	ValueTimeout->value(Timeout);
 	SettingsWindow->show();
 	while(SettingsWindow->shown()) { Fl::wait(); }
 	if(strcmp(DEFAULT_BROWSER, InputBrowser->value())) {
@@ -459,6 +526,18 @@ void ChangeSettings()
 	}
 	SetOffline(ChoiceDownload->value());
 	Debug = CheckDebug->value();
+	if (Debug) {
+		UpstreamTab->activate();
+		DownstreamTab->activate();
+		DebugTab->activate();
+	} else {
+		OutputTabs->value(OutputTab);
+		UpstreamTab->deactivate();
+		DownstreamTab->deactivate();
+		DebugTab->deactivate();
+	}
+	Prefs->set("deletehosts", CheckEraseHosts->value());
+	Prefs->set("timeout", ValueTimeout->value());
 	Prefs->flush();		// Write changes to disk
 }
 
@@ -519,3 +598,58 @@ void Bookmark(int cmd) {		// (0) = get, (!= 0) = set
 		}
 	}
 }
+
+void SetIcon(Fl_Window *w) {
+#ifdef WIN32
+	w->icon((char *)LoadIcon(fl_display, "MAINICON"));
+#elif !defined(__APPLE__)
+	Pixmap p, mask;
+	
+	fl_open_display();
+
+	XpmCreatePixmapFromData(fl_display, DefaultRootWindow(fl_display), stripclub_xpm, &p, &mask, NULL);
+
+	w->icon((char *)p);
+#else
+#warning No icon support for this platform
+#endif
+}
+
+#ifdef WIN32
+
+#include <winsock2.h>
+
+int WinMain(HINSTANCE hInst, HINSTANCE hPrev, LPSTR szCmdLine, int sw) {
+	struct stat filestat;
+        WSADATA wsaData;
+	int ret;
+	char *dummycmd = "stripclub.exe";
+
+	if (stat("stripclub.exe", &filestat)) {	// Can't find the executable, so the working dir must be wrong (sanity check)
+		MOut("Strip Club should not be run from any directory but the location of the executable. Working directory is pointing to the wrong place, perhaps?\n");
+		return 1;
+	}
+
+	DbgOut("Initializing Winsock\n");
+
+	if (WSAStartup(MAKEWORD(2, 2), &wsaData) != 0) {
+		MOut("Couldn't initialize network library,\nis Winsock 2 installed?\n");
+		return 1;
+	}
+
+	DbgOut("Winsock: %d %d %s\n", wsaData.wVersion, wsaData.wHighVersion, wsaData.szDescription);
+
+	ret = main(1, &dummycmd);		// FIXME: Should parse this correctly
+
+	if ( WSACleanup() == SOCKET_ERROR ) {
+		if ( WSAGetLastError() == WSAEINPROGRESS ) {
+			WSACancelBlockingCall();
+			WSACleanup();
+		}
+	}
+
+	return ret;
+}
+
+#endif
+
